@@ -1,12 +1,12 @@
 #entry point for deepforest model
 import os
-import numpy as np
 import pandas as pd
 from skimage import io
 import torch
 
 import pytorch_lightning as pl
 from torch import optim
+import tempfile
 
 from deepforest import utilities
 from deepforest import dataset
@@ -14,13 +14,20 @@ from deepforest import get_data
 from deepforest import model
 from deepforest import predict
 from deepforest import evaluate as evaluate_iou
+from deepforest import visualize
 
 class deepforest(pl.LightningModule):
     """Class for training and predicting tree crowns in RGB images
     """
-    def __init__(self, saved_model=None):
+    def __init__(self, num_classes=1):
+        """
+        Args:
+            num_classes (int): number of classes in the model
+        Returns:
+            self: a deepforest pytorch ligthning module
+        """
         super().__init__()
-        
+                
         # Read config file - if a config file exists in local dir use it,
         # if not use installed.
         if os.path.exists("deepforest_config.yml"):
@@ -38,11 +45,9 @@ class deepforest(pl.LightningModule):
 
         # release version id to flag if release is being used
         self.__release_version__ = None
-
-        if saved_model:
-            utilities.load_saved_model(saved_model)
-        else:
-            self.create_model()
+           
+        self.num_classes = num_classes            
+        self.create_model()
 
     def use_release(self):
         """Use the latest DeepForest model release from github and load model.
@@ -51,24 +56,88 @@ class deepforest(pl.LightningModule):
             model (object): A trained keras model
         """
         # Download latest model from github release
-        release_tag, self.weights = utilities.use_release()
-
+        release_tag, self.release_state_dict = utilities.use_release()
+        self.model.load_state_dict(torch.load(self.release_state_dict,map_location=self.device))
+        
         # load saved model and tag release
         self.__release_version__ = release_tag
         print("Loading pre-built model: {}".format(release_tag))
     
     def create_model(self):
         """Define a deepforest retinanet architecture"""
-        self.backbone = model.load_backbone()
+        self.model = model.create_model(self.num_classes, self.config["nms_thresh"], self.config["score_thresh"])
+    
+    def create_trainer(self, logger=None, callbacks=None, **kwargs):
+        """Create a pytorch ligthning training by reading config files
+        Args:
+            callbacks (list): a list of pytorch-lightning callback classes
+        """
         
-    def predict_image(self, image=None, path=None, return_plot=False,score_threshold=0.01):
+        self.trainer = pl.Trainer(
+            logger=logger,
+            max_epochs=self.config["train"]["epochs"],
+            gpus=self.config["gpus"],
+            checkpoint_callback=False,
+            distributed_backend=self.config["distributed_backend"],
+            fast_dev_run=self.config["train"]["fast_dev_run"],
+            callbacks=callbacks,
+            **kwargs
+        )
+    
+    def save_model(self, path):
+        """Save the trainer checkpoint in user defined path, in order to access in future
+        Arg:
+            Path: the path located the model checkpoint
+        """
+        self.trainer.save_checkpoint(path)
+        
+    def load_dataset(self, csv_file, root_dir=None, augment=False, shuffle=True, batch_size=1):
+        """Create a tree dataset for inference
+        Csv file format is .csv file with the columns "image_path", "xmin","ymin","xmax","ymax" for the image name and bounding box position. 
+        Image_path is the relative filename, not absolute path, which is in the root_dir directory. One bounding box per line. 
+        
+        Args:
+            csv_file: path to csv file 
+            root_dir: directory of images. If none, uses "image_dir" in config
+            augment: Whether to create a training dataset, this activates data augmentations
+        Returns:
+            ds: a pytorch dataset
+        """
+        
+        ds = dataset.TreeDataset(csv_file=csv_file,
+                              root_dir=root_dir,
+                              transforms=dataset.get_transform(augment=augment))
+        
+        data_loader = torch.utils.data.DataLoader(ds,
+                                                  batch_size=batch_size,
+                                                  shuffle=shuffle,
+                                                  collate_fn=utilities.collate_fn,
+                                                  num_workers=self.config["workers"],
+                                                  )
+        
+        return data_loader
+    
+    def train_dataloader(self):
+        loader = self.load_dataset(csv_file=self.config["train"]["csv_file"], root_dir=self.config["train"]["root_dir"], augment=True, shuffle=True, batch_size=self.config["batch_size"])
+        
+        return loader
+    
+    def val_dataloader(self):
+        #only create a val_dataloader is specified in config
+        if self.config["validation"]["csv_file"] is not None:
+            loader = self.load_dataset(csv_file=self.config["validation"]["csv_file"], root_dir=self.config["validation"]["root_dir"], augment=False, shuffle=False, batch_size=self.config["batch_size"])
+        else:
+            loader = None
+            
+        return loader    
+    
+    def predict_image(self, image=None, path=None, return_plot=False):
         """Predict an image with a deepforest model
         
         Args:
             image: a numpy array of a RGB image ranged from 0-255
             path: optional path to read image from disk instead of passing image arg
             return_plot: Return image with plotted detections
-            score_threshold: float [0,1] minimum probability score to return/plot.
         Returns:
             boxes: A pandas dataframe of predictions (Default)
             img: The input with predictions overlaid (Optional)
@@ -81,12 +150,18 @@ class deepforest(pl.LightningModule):
                 raise ValueError("Path expects a string path to image on disk")            
             image = io.imread(path)
         
-        self.backbone.eval()   
-        result = predict.predict_image(model =  self.backbone, image = image, return_plot = return_plot, score_threshold = score_threshold)
+            #Load on GPU is available
+        if torch.cuda.is_available:
+            self.model.to(self.device)   
+            
+        self.model.eval()   
+        
+        #Check if GPU is available and pass image to gpu
+        result = predict.predict_image(model = self.model, image = image, return_plot = return_plot, device=self.device)
         
         return result
                                                  
-    def predict_file(self, csv_file, root_dir, save_dir=None):
+    def predict_file(self, csv_file, root_dir, savedir=None):
         """Create a dataset and predict entire annotation file
         
         Csv file format is .csv file with the columns "image_path", "xmin","ymin","xmax","ymax" for the image name and bounding box position. 
@@ -99,11 +174,10 @@ class deepforest(pl.LightningModule):
         Returns:
             df: pandas dataframe with bounding boxes, label and scores for each image in the csv file
         """
-        
-        self.backbone.eval()
-        df = predict.predict_file(self.backbone, csv_file, root_dir, save_dir)
-
-        return df
+        self.model.eval()
+        result = predict.predict_file(model=self.model, csv_file=csv_file, root_dir=root_dir, savedir=savedir, device=self.device)
+            
+        return result
     
     def predict_tile(self,
                      raster_path=None,
@@ -111,7 +185,6 @@ class deepforest(pl.LightningModule):
                      patch_size=400,
                      patch_overlap=0.05,
                      iou_threshold=0.15,
-                     score_threshold=0,
                      return_plot=False,
                      use_soft_nms = False,
                      sigma = 0.5,
@@ -139,60 +212,33 @@ class deepforest(pl.LightningModule):
             Otherwise a numpy array of predicted bounding boxes, scores and labels
         """
         
-        self.backbone.eval()
-        
+        self.model.eval()
+
         result = predict.predict_tile(
-            model = self.backbone,
+            model = self.model,
             raster_path=raster_path,
             image=image,
             patch_size=patch_size,
             patch_overlap=patch_overlap,
             iou_threshold=iou_threshold,
-            score_threshold=score_threshold,
             return_plot=return_plot,
             use_soft_nms = use_soft_nms,
             sigma = sigma,
-            thresh = thresh)
+            thresh = thresh,
+            device=self.device
+        )
         
         return result
-        
-
-    def load_dataset(self, csv_file, root_dir=None, augment=False):
-        """Create a tree dataset for inference
-        Csv file format is .csv file with the columns "image_path", "xmin","ymin","xmax","ymax" for the image name and bounding box position. 
-        Image_path is the relative filename, not absolute path, which is in the root_dir directory. One bounding box per line. 
-        
-        Args:
-            csv_file: path to csv file 
-            root_dir: directory of images. If none, uses "image_dir" in config
-            augment: Whether to create a training dataset, this activates data augmentations
-        Returns:
-            ds: a pytorch dataset
-        """
-        
-        ds = dataset.TreeDataset(csv_file=csv_file,
-                              root_dir=root_dir,
-                              transforms=dataset.get_transform(augment=augment))
-        
-        train_sampler = torch.utils.data.RandomSampler(ds)
-        train_batch_sampler = torch.utils.data.BatchSampler(
-            train_sampler, self.config["train"]["batch_size"], drop_last=True)
-        
-        data_loader = torch.utils.data.DataLoader(
-            ds, batch_sampler=train_batch_sampler, num_workers=self.config["train"]["workers"],
-            collate_fn=utilities.collate_fn)    
-        
-        return data_loader
 
     def training_step(self, batch, batch_idx):
         """Train on a loaded dataset
         """        
-        images, targets = batch
+        path, images, targets = batch
         
-        loss_dict = self.backbone.forward(images, targets)
+        loss_dict = self.model.forward(images, targets)
         
         #sum of regression and classification loss        
-        losses = sum(loss for loss in loss_dict.values())
+        losses = sum([loss for loss in loss_dict.values()])
         
         return losses
     
@@ -200,53 +246,64 @@ class deepforest(pl.LightningModule):
         """Train on a loaded dataset
 
         """
-        images, targets = batch
+        path, images, targets = batch
+        
+        self.model.train()
+        loss_dict = self.model.forward(images, targets)
         
-        loss_dict = self.backbone.forward(images, targets)
         #sum of regression and classification loss
-        losses = sum(loss for loss in loss_dict.values())
+        losses = sum([loss for loss in loss_dict.values()])
         
+        #Log loss
+        for key, value in loss_dict.items():
+            self.log("val_{}".format(key),value, on_epoch=True)
+                
         return losses
-    
+        
+    def validation_end(self, outputs):
+        avg_loss = torch.stack([x['val_loss'] for x in outputs]).mean()
+        comet_logs = {'val_loss': avg_loss}
+
+        return {'avg_val_loss': avg_loss, 'log': comet_logs}
+        
     def configure_optimizers(self):
-        self.optimizer = optim.SGD(self.backbone.parameters(), lr=self.config["train"]["lr"], momentum=0.9)
+        self.optimizer = optim.SGD(self.model.parameters(), lr=self.config["train"]["lr"], momentum=0.9)
         self.scheduler = torch.optim.lr_scheduler.ReduceLROnPlateau(self.optimizer, mode='min', 
                                                            factor=0.1, patience=10, 
-                                                           verbose=False, threshold=0.0001, 
+                                                           verbose=True, threshold=0.0001, 
                                                            threshold_mode='rel', cooldown=0, 
                                                            min_lr=0, eps=1e-08)
         return self.optimizer
             
-    def evaluate(self, csv_file, root_dir, iou_threshold=0.5, score_threshold=0, show_plot=False, project=False):
+    def evaluate(self, csv_file, root_dir, iou_threshold=None, show_plot=False, savedir=None):
         """Compute intersection-over-union and precision/recall for a given iou_threshold
         
         Args:
             df: a pandas-type dataframe (geopandas is fine) with columns "name","xmin","ymin","xmax","ymax","label", each box in a row
             root_dir: location of files in the dataframe 'name' column.
             iou_threshold: float [0,1] intersection-over-union union between annotation and prediction to be scored true positive
-            score_threshold: float [0,1] minimum probability score to be included in evaluation
-            project: Logical. Whether to project predictions that are in image coordinates (0,0 origin) into the geographic coordinates of the ground truth image. The CRS is take from the image file using rasterio.crs
             show_plot: open a blocking matplotlib window to show plot and annotations, useful for debugging.
+            savedir: optional path dir to save evaluation images
         Returns:
-            results: tuple of (precision, recall) for a given threshold
+            results: dict of ("results", "precision", "recall") for a given threshold
         """
-        predictions = self.predict_file(csv_file, root_dir)
+        self.model.eval()
+        
+        if not self.device.type=="cpu":
+            self.model = self.model.to(self.device)
+            
+        predictions = predict.predict_file(model=self.model, csv_file=csv_file, root_dir=root_dir, savedir=savedir, device=self.device)                
         ground_df = pd.read_csv(csv_file)
         
+        #if no arg for iou_threshold, set as config
+        if iou_threshold is None:
+            iou_threshold = self.config["validation"]["iou_threshold"]
+        
         results = evaluate_iou.evaluate(
             predictions=predictions,
             ground_df=ground_df,
             root_dir=root_dir,
-            project=project,
             iou_threshold=iou_threshold,
-            score_threshold=score_threshold,
             show_plot=show_plot)
         
-        return results
-
-    def save(self):
-        pass
-    
-    def load(self):
-        pass
-    
\ No newline at end of file
+        return results
\ No newline at end of file
