@@ -1,9 +1,9 @@
 #test dataset model
 from deepforest import get_data
 from deepforest import dataset
-from torch.utils.data import DataLoader
 import os
 import pytest
+import pandas as pd
 
 def test_TreeDataset():
     csv_file = get_data("example.csv")
@@ -11,10 +11,13 @@ def test_TreeDataset():
     ds = dataset.TreeDataset(csv_file=csv_file,
                              root_dir=root_dir,
                              transforms=None)
-
+    raw_data = pd.read_csv(csv_file)
+    
+    assert len(ds) == len(raw_data.image_path.unique())
+    
     for i in range(len(ds)):
         #Between 0 and 1
-        image, targets = ds[i]
+        path, image, targets = ds[i]
         assert image.max() <= 1
         assert image.min() >= 0
         assert targets["boxes"].shape == (79, 4)
@@ -31,7 +34,7 @@ def test_TreeDataset_transform(augment):
 
     for i in range(len(ds)):
         #Between 0 and 1
-        image, targets = ds[i]
+        path, image, targets = ds[i]
         assert image.max() <= 1
         assert image.min() >= 0
         assert targets["boxes"].shape == (79, 4)
