@@ -9,76 +9,80 @@ import torch
 
 from deepforest import main
 from deepforest import get_data
-from deepforest.utilities import collate_fn
-from deepforest.callbacks import evaluate_callback
+from deepforest import model
 
 from pytorch_lightning import Trainer
 from pytorch_lightning.callbacks import Callback
 
+#Import release model from global script to avoid thrasing github during testing. Just download once.
+from .conftest import download_release
 
 @pytest.fixture()
-def m():
+def m(download_release):
     m = main.deepforest()
-    m.config["epochs"] = 1
+    m.config["train"]["csv_file"] = get_data("example.csv") 
+    m.config["train"]["root_dir"] = os.path.dirname(get_data("example.csv"))
+    m.config["train"]["fast_dev_run"] = True
     m.config["batch_size"] = 2
-    
-    return m
-
-@pytest.fixture()
-def trained_model():
-    m = main.deepforest()
         
+    m.config["validation"]["csv_file"] = get_data("example.csv") 
+    m.config["validation"]["root_dir"] = os.path.dirname(get_data("example.csv"))
+
+    m.create_trainer()
+    m.use_release()
+    
     return m
 
 def test_main():
     from deepforest import main
 
-def test_train(m):    
-    csv_file = get_data("example.csv") 
-    root_dir = os.path.dirname(csv_file)
-    train_ds = m.load_dataset(csv_file, root_dir=root_dir)
-    trainer = Trainer(fast_dev_run=True)
-    trainer.fit(m, train_ds)
+def test_train(m):
+    m.trainer.fit(m)
 
+def test_train_no_validation(m):
+    m.config["validation"]["csv_file"] = None
+    m.config["validation"]["root_dir"] = None  
+    m.trainer.fit(m)
+    
 def test_predict_image_empty(m):
     image = np.random.random((400,400,3))
     prediction = m.predict_image(image = image)
     
     assert prediction is None
     
-def test_predict_image_fromfile(trained_model):
+def test_predict_image_fromfile(m):
     path = get_data(path="2019_YELL_2_528000_4978000_image_crop2.png")
-    prediction = trained_model.predict_image(path = path)
+    prediction = m.predict_image(path = path)
     
     assert isinstance(prediction, pd.DataFrame)
     assert set(prediction.columns) == {"xmin","ymin","xmax","ymax","label","scores"}
 
-def test_predict_image_fromarray(trained_model):
+def test_predict_image_fromarray(m):
     image = get_data(path="2019_YELL_2_528000_4978000_image_crop2.png")
     image = io.imread(image)
-    prediction = trained_model.predict_image(image = image)
+    prediction = m.predict_image(image = image)
     
     assert isinstance(prediction, pd.DataFrame)
     assert set(prediction.columns) == {"xmin","ymin","xmax","ymax","label","scores"}
 
-def test_predict_return_plot(trained_model):
+def test_predict_return_plot(m):
     image = get_data(path="2019_YELL_2_528000_4978000_image_crop2.png")
     image = io.imread(image)
-    plot = trained_model.predict_image(image = image, return_plot=True)
+    plot = m.predict_image(image = image, return_plot=True)
     assert not isinstance(plot, pd.DataFrame)
             
-def test_predict_file(trained_model, tmpdir):
+def test_predict_file(m, tmpdir):
     csv_file = get_data("example.csv")
-    df = trained_model.predict_file(csv_file, root_dir = os.path.dirname(csv_file), save_dir=tmpdir)
+    df = m.predict_file(csv_file, root_dir = os.path.dirname(csv_file), savedir=tmpdir)
     assert set(df.columns) == {"xmin","ymin","xmax","ymax","label","scores","image_path"}
     
     printed_plots = glob.glob("{}/*.png".format(tmpdir))
     assert len(printed_plots) == 1
 
-def test_predict_tile(trained_model):
+def test_predict_tile(m):
     #test raster prediction 
     raster_path = get_data(path= 'OSBS_029.tif')
-    prediction = trained_model.predict_tile(raster_path = raster_path,
+    prediction = m.predict_tile(raster_path = raster_path,
                                             patch_size = 300,
                                             patch_overlap = 0.5,
                                             return_plot = False)
@@ -87,7 +91,7 @@ def test_predict_tile(trained_model):
     assert not prediction.empty
 
     #test soft-nms method
-    soft_nms_pred = trained_model.predict_tile(raster_path = raster_path,
+    soft_nms_pred = m.predict_tile(raster_path = raster_path,
                                             patch_size = 300,
                                             patch_overlap = 0.5,
                                             return_plot = False,
@@ -98,23 +102,28 @@ def test_predict_tile(trained_model):
 
     #test predict numpy image
     image = io.imread(raster_path)
-    prediction = trained_model.predict_tile(image = image,
+    prediction = m.predict_tile(image = image,
                                 patch_size = 300,
                                 patch_overlap = 0.5,
                                 return_plot = False)
     assert not prediction.empty
 
     # Test no non-max suppression
-    prediction = trained_model.predict_tile(raster_path = raster_path,
+    prediction = m.predict_tile(raster_path = raster_path,
                                        patch_size=300,
                                        patch_overlap=0,
                                        return_plot=False)
     assert not prediction.empty
     
 def test_evaluate(m):
-    csv_file = get_data("example.csv")
+    csv_file = get_data("OSBS_029.csv")
     root_dir = os.path.dirname(csv_file)
-    precision, recall = m.evaluate(csv_file, root_dir, iou_threshold = 0.5)
+    
+    results = m.evaluate(csv_file, root_dir, iou_threshold = 0.4, show_plot=True)
+    
+    #Does this make reasonable predictions, we know the model works.
+    assert np.round(results["precision"],2) == 0.43
+    assert np.round(results["recall"],2) == 0.74
     
 def test_train_callbacks(m):
     csv_file = get_data("example.csv") 
@@ -135,13 +144,20 @@ def test_train_callbacks(m):
     trainer = Trainer(callbacks=[MyPrintingCallback()])
     
     trainer = Trainer(fast_dev_run=True)
-    trainer.fit(m, train_ds)    
+    trainer.fit(m, train_ds)
     
-def test_precision_recall_callbacks(m):
-    csv_file = get_data("example.csv") 
-    root_dir = os.path.dirname(csv_file)
-    train_ds = m.load_dataset(csv_file, root_dir=root_dir)
-    eval_callback = evaluate_callback(csv_file, root_dir)
-        
-    trainer = Trainer(callbacks=[eval_callback], limit_train_batches=0.01, limit_val_batches=0.01, max_epochs= 1)
-    trainer.fit(m, train_ds)    
\ No newline at end of file
+def test_save_and_reload(m, tmpdir):
+    img_path = get_data(path="2019_YELL_2_528000_4978000_image_crop2.png")
+    m.trainer.fit(m)
+    
+    #save the prediction dataframe after training and compare with prediction after reload checkpoint 
+    pred_after_train = m.predict_image(path = img_path)
+    m.save_model("{}/checkpoint.pl".format(tmpdir))
+    
+    #reload the checkpoint to model object
+    after = main.deepforest.load_from_checkpoint("{}/checkpoint.pl".format(tmpdir))
+    pred_after_reload = after.predict_image(path = img_path)
+
+    assert not pred_after_train.empty
+    assert not pred_after_reload.empty
+    pd.testing.assert_frame_equal(pred_after_train,pred_after_reload)
